@@ -15,4 +15,18 @@ function user_create_send_notification_button() {
     this.submit( null, null, function(data){
         data.resetpw = true;
     })
-}
\ No newline at end of file
+}
+
+/**
+ * dismiss form
+ */
+ function dismiss_button() {
+    this.close();
+}
+
+/**
+ * submit form
+ */
+function submit_button() {
+    this.submit();
+}
